//------------------------------------------------------------------------------
// File: GrabTsForBD,cpp
//   Implementation of GrabTsForBD.dll
//------------------------------------------------------------------------------

#include <streams.h>		// DirectShow (includes windows.h)
#include <initguid.h>		// declares DEFINE_GUID to declare an EXTERN_C const.
#include "IGrabTsForBD.h"	// declares myself


// TS data buffer size
//
#define TS_BLK_SIZE (0x10000)
#define TS_BUF_SIZE (1024*1024*32) // = 40Mbps/8bit*6sec

// setup data - allows the self-registration to work.
//
const AMOVIESETUP_MEDIATYPE sudPinTypes =
{
	&MEDIATYPE_NULL,		// clsMajorType
	&MEDIASUBTYPE_NULL		// clsMinorType
} ;

const AMOVIESETUP_PIN psudPins[] =
{
	{
		L"Input",			// strName
		FALSE,				// bRendered
		FALSE,				// bOutput
		FALSE,				// bZero
		FALSE,				// bMany
		&CLSID_NULL,		// clsConnectsToFilter
		L"",				// strConnectsToPin
		1,					// nTypes
		&sudPinTypes		// lpTypes
	},
	{
		L"Output",			// strName
		FALSE,				// bRendered
		TRUE,				// bOutput
		FALSE,				// bZero
		FALSE,				// bMany
		&CLSID_NULL,		// clsConnectsToFilter
		L"",				// strConnectsToPin
		1,					// nTypes
		&sudPinTypes		// lpTypes
	}
} ;

const AMOVIESETUP_FILTER sudGrabTsForBD =
{
	&CLSID_GrabTsForBD,		// clsID
	L"GrabTsForBD",			// strName
	MERIT_DO_NOT_USE,		// dwMerit
	2,						// nPins
	psudPins				// lpPin
} ;


// CGrabTsForBD class
//
class CGrabTsForBD :
	public CTransInPlaceFilter , public IGrabTsForBD
{
public:
	static CUnknown *WINAPI CreateInstance( LPUNKNOWN punk, HRESULT *phr ) ;
	DECLARE_IUNKNOWN;
private:
	// Constructor
	CGrabTsForBD( TCHAR *tszName, LPUNKNOWN punk, HRESULT *phr )
		: CTransInPlaceFilter ( tszName, punk, CLSID_GrabTsForBD, phr )
	{
		m_nPush = m_nPull = 0 ;
		InitializeCriticalSection( &m_cs ) ;
	}
	// Destructor
	~CGrabTsForBD()
	{
		DeleteCriticalSection( &m_cs ) ;
	}
	// Overrides
	HRESULT Transform(IMediaSample *pSample) ;
	HRESULT CheckInputType(const CMediaType* mtIn) { return S_OK; }
	STDMETHODIMP NonDelegatingQueryInterface(REFIID riid, void ** ppv);
	// IGrabTsForBD - private interface to get TS stream
	STDMETHODIMP get_ReadyCount( DWORD* pdwRemain ) ;
	STDMETHODIMP get_TsStream( BYTE** ppDst, DWORD* pdwSize, DWORD* pdwRemain ) ;
	STDMETHODIMP purge_TsStream( void ) ;
	// exclusive access to TS data buffer
	CRITICAL_SECTION m_cs ;
	void Lock( void ) { EnterCriticalSection( &m_cs ) ; }
	void UnLock( void ) { LeaveCriticalSection( &m_cs ) ; }
	// TS data buffer ( simple ring buffer )
	int m_nPush ;
	int m_nPull ;
	BYTE m_ayBuf[TS_BUF_SIZE] ;
} ;

// Transform function
//
HRESULT CGrabTsForBD::Transform(IMediaSample *pSample)
{
	// copy data to ring buffer
	long lBytes = pSample->GetActualDataLength() ;
	BYTE* pBuf ;
	pSample->GetPointer(&pBuf);
	Lock() ;
	if ( m_nPush + lBytes <= TS_BUF_SIZE ) {
		memcpy( m_ayBuf + m_nPush, pBuf, lBytes ) ;
	}
	else {
		memcpy( m_ayBuf + m_nPush, pBuf, TS_BUF_SIZE - m_nPush ) ;
		memcpy( m_ayBuf, pBuf + TS_BUF_SIZE - m_nPush, m_nPush + lBytes - TS_BUF_SIZE ) ;
	}
	m_nPush = ( m_nPush + lBytes ) % TS_BUF_SIZE ;
	UnLock() ;

	// DO NOT pass data to downstream
	#ifdef _DEBUG
	return S_OK ;
	#else
	return S_FALSE ;
	#endif
}

// NonDelegatingQueryInterface
//
STDMETHODIMP CGrabTsForBD::NonDelegatingQueryInterface(REFIID riid, void **ppv)
{
	CheckPointer(ppv,E_POINTER);
	// return private interface
	if (riid == IID_IGrabTsForBD) {
		return GetInterface((IGrabTsForBD *) this, ppv);
	}
	// call base class
	else return CTransInPlaceFilter::NonDelegatingQueryInterface(riid, ppv);
}

// Get number of available TS data blocks (IGrabTsForBD)
//
HRESULT CGrabTsForBD::get_ReadyCount( DWORD* pdwRemain )
{
	if ( pdwRemain ) {
		Lock() ;
		*pdwRemain = ( ( m_nPush + TS_BUF_SIZE - m_nPull ) % TS_BUF_SIZE ) / TS_BLK_SIZE ;
		UnLock() ;
	}
	return NOERROR ;
}

// Get one TS data block in ring buffer (IGrabTsForBD)
//
HRESULT CGrabTsForBD::get_TsStream( BYTE** ppDst, DWORD* pdwSize, DWORD* pdwRemain )
{
	Lock() ;
	int nPush = m_nPush ;
	UnLock() ;
	if ( ( ( nPush + TS_BUF_SIZE - m_nPull ) % TS_BUF_SIZE ) >= TS_BLK_SIZE ) {
		*ppDst = &m_ayBuf[m_nPull] ;
		*pdwSize = TS_BLK_SIZE ;
		m_nPull = ( m_nPull + TS_BLK_SIZE ) % TS_BUF_SIZE ;
	}
	else {
		*ppDst = &m_ayBuf[m_nPull] ;
		*pdwSize = 0 ;
	}
	if ( pdwRemain ) {
		*pdwRemain = ( ( nPush + TS_BUF_SIZE - m_nPull ) % TS_BUF_SIZE ) / TS_BLK_SIZE ;
	}
	return NOERROR ;
}

// Purge TS data in ring buffer (IGrabTsForBD)
//
HRESULT CGrabTsForBD::purge_TsStream( void )
{
	Lock() ;
	m_nPush = m_nPull = 0 ;
	UnLock() ;
	return NOERROR ;
}



// DllEntryPoint
//
extern "C" BOOL WINAPI DllEntryPoint(HINSTANCE, ULONG, LPVOID);
BOOL APIENTRY DllMain( HANDLE hModule, DWORD dwReason, LPVOID lpReserved )
{
	return DllEntryPoint( (HINSTANCE)(hModule), dwReason, lpReserved ) ;
}


// Exported entry points to register/unregister
//
STDAPI DllRegisterServer()
{
	return AMovieDllRegisterServer2( TRUE ) ;
}

STDAPI DllUnregisterServer()
{
	return AMovieDllRegisterServer2( FALSE ) ;
}


// Class factory template
//
CFactoryTemplate g_Templates[] =
{
	{
		L"GrabTsForBD",
		&CLSID_GrabTsForBD,
		CGrabTsForBD::CreateInstance,
		NULL,
		&sudGrabTsForBD
	}
} ;
int g_cTemplates = sizeof(g_Templates)/sizeof(g_Templates[0]);


// CreateInstance
//
CUnknown * WINAPI CGrabTsForBD::CreateInstance( LPUNKNOWN punk, HRESULT *phr )
{
	CheckPointer( phr, NULL ) ;
	CGrabTsForBD *pNewObject =
		new CGrabTsForBD( NAME("Grab TS stream filter for BonDriver_DW2104"), punk, phr ) ;
	if ( pNewObject == NULL ) {
		*phr = E_OUTOFMEMORY ;
	}
	return pNewObject ;
}

